# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django_pgjson.fields
from django.conf import settings
import geokey.core.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0005_auto_20150202_1041'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Subset',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(null=True, blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('filters', django_pgjson.fields.JsonBField(null=True, blank=True)),
                ('where_clause', models.TextField(null=True, blank=True)),
                ('creator', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
                ('project', models.ForeignKey(related_name='subsets', to='projects.Project')),
            ],
            options={
            },
            bases=(geokey.core.mixins.FilterMixin, models.Model),
        ),
    ]



class FilterMixin(object):
    def save(self, *args, **kwargs):
        """
        Overwrites save to implement integrity ensurance.
        """
        self.where_clause = None
        if self.filters is not None:
            queries = []

            for key in self.filters:
                category = self.project.categories.get(pk=key)
                queries.append(category.get_query(self.filters[key]))

            if len(queries) > 0:
                query = ' OR '.join(queries)
                self.where_clause = query
            else:
                self.where_clause = 'FALSE'

        super(FilterMixin, self).save(*args, **kwargs)



from django import template

register = template.Library()


@register.simple_tag
def more_link_text(count, singular, plural, minus=5):
    return 'Show {more_count} more {label}'.format(
        more_count=count-minus,
        label=(plural if count-minus != 1 else singular)
    )


@register.simple_tag
def minus(count, to_sub):
    return count - to_sub


@register.simple_tag
def plus(count, to_add):
    return count + to_add



import factory

from geokey.users.tests.model_factories import UserF
from geokey.projects.tests.model_factories import ProjectF

from ..models import Subset


class SubsetFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Subset

    name = factory.Sequence(lambda n: 'project %d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    creator = factory.SubFactory(UserF)
    project = factory.SubFactory(ProjectF)



from rest_framework.serializers import ModelSerializer
from .models import Subset


class SubsetSerializer(ModelSerializer):
    """
    Serialises Subset model instances
    """
    class Meta:
        model = Subset
        depth = 1
        fields = (
            'id', 'name', 'description', 'creator'
        )
